#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Project       FreeImagePy
# file name:    FreeImagePy.py
# Version:      1.0.0
#
# DESCRIPTION
# FreeImage python bind
#
# Developer:
# Michele Petrazzo <michele.petrazzo@unipex.it>
#
# History :
# creation: 08/2005 - Michele Petrazzo
# 
# License:
#         GNU General Public License (GPL)
#         FreeImage Public License (FIPL)
#         For more info see LICENSE.txt
#
# Copyright (C) 2005  Unipex s.r.l.,  All Rights Reserved.
# Via Vittorio Veneto 83/A
# 33050 Gonars (UD) - Italy
# tel. +39 0432 931511 - fax +39 0432 931378
# www.unipex.it - michele.petrazzo@unipex.it
#

import ctypes as C 
import os, os.path, tempfile, sys, time

#Load the constants

from constants import *

"""
Extension to type
See loadAndSave function

string: type, args and extension

"""
extToType = dict(
             tiff       = (FIF_TIFF, TIFF_DEFAULT,          '.tif'),
             tiffg3     = (FIF_TIFF, TIFF_CCITTFAX3,        '.tif'),
             tiffg4     = (FIF_TIFF, TIFF_CCITTFAX4,        '.tif'),
             tiffno     = (FIF_TIFF, TIFF_NONE,             '.tif'),
             
             jpeg       = (FIF_JPEG, JPEG_DEFAULT,          '.jpg'),
             jpegfa     = (FIF_JPEG, JPEG_FAST,             '.jpg'),
             jpegac     = (FIF_JPEG, JPEG_ACCURATE,         '.jpg'),
             jpegsu     = (FIF_JPEG, JPEG_QUALITYSUPERB,    '.jpg'),
             jpeggo     = (FIF_JPEG, JPEG_QUALITYGOOD,      '.jpg'),
             jpegav     = (FIF_JPEG, JPEG_QUALITYAVERAGE,   '.jpg'),
             jpegba     = (FIF_JPEG, JPEG_QUALITYBAD,       '.jpg'),
             
             png        = (FIF_PNG,  0,                     '.png'),
             
             bmp        = (FIF_BMP,  0,                     '.bmp'),
            )

class freeimage(object):
    
    """ FreeImage class wrapper
        
        The method with the first line uppercase are internal library methods
        the other one are internal method for help the user
    """
    
    #Enable the message output
    if sys.platform == 'win32':
        MessageOutput = C.WINFUNCTYPE(VOID, C.c_int, C.c_char_p)
    else:
        MessageOutput = C.CFUNCTYPE(VOID, C.c_int, C.c_char_p)
    
    def printErrors(self, imageFormat, message):
        print 'Error returned', imageFormat, message
    
    #Structures
    class __RGBQUAD(C.Structure):
        _fields_ = [ ("rgbBlue",     BYTE),
                     ("rgbGreen",    BYTE),
                     ("rgbRed",      BYTE),
                     ("rgbReserved", BYTE) ]
    
    class __FIBITMAP(C.Structure):
        _fields_ = [ ("data",   C.POINTER(VOID)) ]
    
    class __FITAG(C.Structure):
        _fields_ = [ ("data",   VOID), ]        
    
    class __FIMETADATA(C.Structure):
        _fields_ = [ ("data", VOID ), ]        
    
    class __PBITMAPINFOHEADER(C.Structure):
        _fields_ = [ ("biSize",             DWORD ),
                     ("biWidth",            LONG  ),
                     ("biHeight",           LONG  ),
                     ("biPlanes",           WORD  ),
                     ("biBitCount",         WORD  ),
                     ("biCompression",      DWORD ),
                     ("biSizeImage",        DWORD ),
                     ("biXPelsPerMeter",    LONG  ),
                     ("biYPelsPerMeter",    LONG  ),
                     ("biClrUsed",          DWORD ),
                     ("biClrImportant",     DWORD ), ]
    
    class __internlLibrary(object):
        """ Class for the real library load
        """
        def __init__(self, libraryName):
            """ Initialize the library and set the self.init value
                to error if an exception is raise
            """
            self.__init = (0, 'ok')
            if sys.platform == 'win32':
                if not libraryName: 
                    dirPath = os.path.dirname(__file__)
                    libraryName = os.path.join(dirPath, 'FreeImage.dll')
                if not os.path.exists(libraryName):
                    raise FreeImagePy_LibraryNotFound, \
                        "I cannot find the library at %s. Did you pass me a vaid path?" % libraryName
                try:
                    self.lib = C.windll.LoadLibrary(libraryName)
                except Exception, ex:
                    self.__init = (1, str(ex))
                
            else:
                if not libraryName: libraryName = '/usr/lib/libfreeimage.so.3'
                try:
                    self.lib = C.cdll.LoadLibrary(libraryName)
                    self.lib.FreeImage_Initialise(False)
                except Exception, ex:
                    self.__init = (1, str(ex))
            self.__init = (0, 'Library %s. Loaded at: %s' % (str(self.lib), libraryName))
                
        def setBind(self, function):
            """ Call me when you want to set a new bind
            """
            restype = None
            name, add = function[0:2]
            if len(function) == 4:
                restype = function[3]
            
            #The real name
            nameToBind = name.split('_', 1)[1]
            
            #Control the platform where we are working
            if sys.platform == 'win32':
                functionName = getattr(self.lib, '_%s%s' % (name, add) ) 
            else:
                functionName = getattr(self.lib, name)
            
            setattr( self, nameToBind, functionName )
            
            if restype: functionName.restype = restype
            
        def getStatus(self):
            """ Return the current status
                Now I'm used only for cotrol the correct initialialize, but
                in the future...
            """
            return self.__init
        
    def __init__(self, libraryName=None):
        """
        """
        
        # Used for test
        self.__num = 0
        # Used for test
        
        self.initCalled = 0
        self.__lib = self.__internlLibrary(libraryName)
        if self.__lib.getStatus()[0]:
            return 'Error: %s' % str(self.__lib.getStatus())
        
        #Functions list that now acutally wrap. The third value are the return
        #type, if it exists, or if I'm able to translate from C code :)
        self.__lstFunction = ( 
        
            #General funtions
            ('FreeImage_Initialise',            '@4'), 
            ('FreeImage_DeInitialise',          '@0'),
            ('FreeImage_GetVersion',            '@0', None, C.c_char_p), 
            ('FreeImage_GetCopyrightMessage',   '@0', None, C.c_char_p), 
            ('FreeImage_SetOutputMessage',      '@4'),
            
            #Bitmap management functions
            ('FreeImage_Allocate',      '@24', COL_1TO32),
            ('FreeImage_AllocateT',     '@28'),
            ('FreeImage_Load',          '@12'),
            ('FreeImage_Save',          '@16'),
            ('FreeImage_Clone',         '@4'),
            ('FreeImage_Unload',        '@4'),
            
            #Bitmap information
            ('FreeImage_GetImageType',          '@4'),
            ('FreeImage_GetColorsUsed',         '@4', COL_1TO32 ),
            ('FreeImage_GetBPP',                '@4'),
            ('FreeImage_GetWidth',              '@4'),
            ('FreeImage_GetHeight',             '@4'),
            ('FreeImage_GetBits',               '@4', None, C.POINTER(BYTE)),
            ('FreeImage_GetLine',               '@4'),
            ('FreeImage_GetPitch',              '@4'),
            ('FreeImage_GetDIBSize',            '@4'),
            ('FreeImage_GetPalette',            '@4', COL_1TO32, 
                C.POINTER(self.__RGBQUAD) ),
            ('FreeImage_GetDotsPerMeterX',      '@4'),
            ('FreeImage_GetDotsPerMeterY',      '@4'),
            ('FreeImage_SetDotsPerMeterX',      '@8'), 
            ('FreeImage_SetDotsPerMeterY',      '@8'),
            ('FreeImage_GetInfoHeader',         '@4', COL_1TO32,
                C.POINTER(self.__PBITMAPINFOHEADER)),
            ('FreeImage_GetColorType',          '@4', COL_1TO32 ),
            ('FreeImage_GetRedMask',            '@4', COL_1TO32 ),
            ('FreeImage_GetGreenMask',          '@4', COL_1TO32 ),
            ('FreeImage_GetBlueMask',           '@4', COL_1TO32 ),
            ('FreeImage_GetTransparencyCount',  '@4', COL_1TO32 ),
            ('FreeImage_GetTransparencyTable',  '@4', (COL_8,), C.POINTER(BYTE)),
            ('FreeImage_SetTransparencyTable',  '@12', (COL_8,) ),
            ('FreeImage_SetTransparent',        '@8', (COL_8, COL_32) ),
            ('FreeImage_HasBackgroundColor',    '@4', (COL_8, COL_24, COL_32) ),
            ('FreeImage_GetBackgroundColor',    '@8', (COL_8, COL_24, COL_32),
                C.POINTER(self.__RGBQUAD) ),
            ('FreeImage_SetBackgroundColor',    '@8', (COL_8, COL_24, COL_32) ),
            
            #Filetype functions
            ('FreeImage_GetFileType',       '@8'), 
            
            #Pixel access
            ('FreeImage_GetBits',       '@4',  None, BYTE),
            ('FreeImage_GetScanLine',   '@8',  None, BYTE),
            ('FreeImage_GetPixelIndex', '@16', (COL_1, COL_4, COL_8) ),
            ('FreeImage_GetPixelColor', '@16', (COL_16, COL_24, COL_32) ),
            ('FreeImage_SetPixelIndex', '@16', (COL_1, COL_4, COL_8) ),
            ('FreeImage_SetPixelColor', '@16', (COL_1, COL_4, COL_8) ),

            #Conversion / Trasformation
            ('FreeImage_ConvertTo4Bits',        '@4', COL_1TO32),
            ('FreeImage_ConvertTo8Bits',        '@4', COL_1TO32),
            ('FreeImage_ConvertTo16Bits555',    '@4', COL_1TO32),
            ('FreeImage_ConvertTo16Bits565',    '@4', COL_1TO32),
            ('FreeImage_ConvertTo24Bits',       '@4', COL_1TO32),
            ('FreeImage_ConvertTo32Bits',       '@4', COL_1TO32),
            ('FreeImage_ColorQuantize',         '@8', (COL_24,)),
            ('FreeImage_ColorQuantizeEx',       '@20', (COL_24,)),
            ('FreeImage_Threshold',             '@8', COL_1TO32),
            ('FreeImage_Dither',                '@8', COL_1TO32),
            ('FreeImage_ConvertFromRawBits',    '@36', COL_1TO32),
            ('FreeImage_ConvertToRawBits',      '@32', COL_1TO32),
            ('FreeImage_ConvertToStandardType', '@8'),
            ('FreeImage_ConvertToType',         '@12'),
            ('FreeImage_ConvertToRGBF',         '@4', (COL_24, COL_32,)),
            

            #Plugin
            ('FreeImage_GetFIFCount',               '@0'),
            ('FreeImage_SetPluginEnabled',          '@8'),
            ('FreeImage_FIFSupportsReading',        '@4'), 
            ('FreeImage_GetFIFFromFilename',        '@4'),
            ('FreeImage_FIFSupportsExportBPP',      '@8'),
            ('FreeImage_FIFSupportsExportType',     '@8'),
            ('FreeImage_FIFSupportsICCProfiles',    '@4'),
            ('FreeImage_FIFSupportsWriting',        '@4'),
            ('FreeImage_IsPluginEnabled',           '@4'),
            ('FreeImage_RegisterExternalPlugin',    '@20'),
            ('FreeImage_RegisterLocalPlugin',       '@20'),           
            ('FreeImage_GetFIFDescription',         '@4', None, C.c_char_p),
            ('FreeImage_GetFIFExtensionList',       '@4', None, C.c_char_p),
            ('FreeImage_GetFIFFromFormat',          '@4', None, C.c_char_p),
            ('FreeImage_GetFIFFromMime',            '@4', None, C.c_char_p),
            ('FreeImage_GetFIFMimeType',            '@4', None, C.c_char_p),
            ('FreeImage_GetFIFRegExpr',             '@4', None, C.c_char_p),
            ('FreeImage_GetFormatFromFIF',          '@4', None, C.c_char_p),
            
            #Upsampling / downsampling
            ('FreeImage_Rescale',       '@16', (COL_8, COL_24, COL_32) ),
            
            #Rotation and flipping
            ('FreeImage_RotateClassic', '@12', COL_1TO32),
            ('FreeImage_RotateEx',      '@48', (COL_8, COL_24, COL_32), ),

            
            #Color manipulation
            ('FreeImage_AdjustBrightness',  '@12'), 
            ('FreeImage_AdjustCurve',       '@12'),
            ('FreeImage_AdjustGamma',       '@12'),
            ('FreeImage_AdjustContrast',    '@12'),
            ('FreeImage_GetHistogram',      '@12'),
            ('FreeImage_Invert',            '@4'), 
            ('FreeImage_GetChannel',        '@8'),
            ('FreeImage_SetChannel',        '@12'),
            ('FreeImage_GetComplexChannel', '@8'),
            ('FreeImage_SetComplexChannel', '@12'),
            
            #Multipage
            ('FreeImage_OpenMultiBitmap',       '@24'), 
            ('FreeImage_AppendPage',            '@8'), 
            ('FreeImage_CloseMultiBitmap',      '@8'), 
            ('FreeImage_GetPageCount',          '@4'),
            ('FreeImage_LockPage',              '@8'), 
            ('FreeImage_UnlockPage',            '@12'),
            ('FreeImage_InsertPage',            '@12'),
            ('FreeImage_DeletePage',            '@8'),
            ('FreeImage_MovePage',              '@12'),
            ('FreeImage_GetLockedPageNumbers',  '@12'),
            
            #Tag
            ('FreeImage_GetTagValue',       '@4'), 
            ('FreeImage_GetTagDescription', '@4',  None, C.c_char_p), 
            ('FreeImage_TagToString',       '@12', None, C.c_char_p),
            ('FreeImage_GetTagCount',       '@4',  None, DWORD),
            
            #Metadata
            ('FreeImage_GetMetadata',       '@16'), 
            ('FreeImage_GetTagID',          '@4', None, C.c_char_p),
            ('FreeImage_GetMetadataCount',  '@8', None, DWORD),
            
            ('FreeImage_IsLittleEndian',    '@0')
            
            # --------------- This functions don't work yet :(
            
            #FreeImage_GetInfo
            #I don't know how to use the FreeImage_GetInfo return value
            
            #All handle functions...
            #FreeImage_LoadFromHandle
            #FreeImage_GetFileTypeFromHandle
            
            
        )
        
        for function in self.__lstFunction:
            try:
                self.__lib.setBind(function)
            except AttributeError, ex:
                print "Error on bind %s. I cannot find this function" % function[0]

        self.funct_printErrors = self.MessageOutput(self.printErrors)
        self.__lib.SetOutputMessage(self.funct_printErrors)
    
    # ------------------ #
    #  General funtions  #
    # ------------------ #
    
    def GetCopyrightMessage(self):
        """ Returns a string containing a standard copyright message you 
            can show in your program.
        """
        return self.__lib.GetCopyrightMessage()
    
    def GetVersion(self):
        """ Returns a string containing the current version of the DLL.
        """
        return self.__lib.GetVersion()
    
    def Initialise(self, loadPlugin = False):
        """ Initialises the library. When the loadPlugin 
        parameter is TRUE, FreeImage wont make use of external plugins.
        """
        return self.__lib.Initialise(loadPlugin)

    def DeInitialise(self):
        """ Deinitialises the library.
        """
        if sys.platform == 'win32': return True
        return self.__lib.DeInitialise()
        
    def Allocate(self, widht, height, bpp, 
                redMask=False, greenMask=False, blueMask=False):
        """ If you want to create a new bitmap in memory from scratch, 
            without loading a pre-made bitmap from disc, you use this function.
        """
        return self.__lib.Allocate(widht, height, bpp, 
                    redMask, greenMask, blueMask)
    
    def AllocateT(self, type, width, height, bpp=8, red_mask=0, green_mask=0, 
            blue_mask=0):
        """
        """
        self.__ctrlColorUsed()
        return self.__lib.AllocateT(type, width, height, bpp, red_mask, 
                    green_mask, blue_mask)
    
    def Clone(self, bitmap):
        """ Makes an exact reproduction of an existing bitmap
        """
        self.__ctrlColorUsed()
        return self.__lib.Clone(bitmap)
    
    def Load(self, type, fileName, flags=False):
        """ This function decodes a bitmap, allocates 
            memory for it and then returns it as a FIBITMAP.
        """
        return self.__lib.Load(type, fileName, flags)

    def Save(self, type, bitmap, fileName, flags=False):
        """ This function saves a previously loaded FIBITMAP to a file.
        """
        return self.__lib.Save(type, bitmap, fileName, flags)
        
    def Unload(self, bitmap):
        """ Deletes a previously loaded FIBITMAP from memory. 
            
            AAA -> You always need to call this function once you are done 
                   with a bitmap, or you will have a memory leak
        """
        return self.__lib.Unload(bitmap)

    # ------------------- #
    #  MultiPage methods  #
    # ------------------- #
    
    def OpenMultiBitmap(self, format, fileName, create, readOnly=False,
                        keepInMemory=False, flags=False):
        
        """ flags is to enable or disable a feature into a plugin
        """ 
        return self.__lib.OpenMultiBitmap(format, fileName, create, readOnly,
                        keepInMemory, flags)
        
    def GetPageCount(self, bitmap):
        """ Return how many pages are present into this bitmap
        """
        return self.__lib.GetPageCount(bitmap)
    
    def LockPage(self, bitmap, page):
        """ Lock and return a locked bitmpa into a multipage bitmap
        """
        return self.__lib.LockPage(bitmap, page)
    
    def UnlockPage(self, multiBitmap, bitmap, modified=False):
        """ UnLock a page into a multipage bitmap
            if modified, the engine update the bitmap with that passed
        """
        return self.__lib.UnlockPage(multiBitmap, bitmap, modified)
    
    def AppendPage(self, bitmap, data):
        """ Append data page to the bitmap's end
        """
        return self.__lib.AppendPage(bitmap, data)

    def CloseMultiBitmap(self, bitmap, flags=False):
        """ Flags is to enable or disable a feature into a plugin
        """
        return self.__lib.CloseMultiBitmap(bitmap, flags)

    def DeletePage(self, bitmap, page):
        """ Deletes the page on the given position.
        """
        return self.__lib.DeletePage(bitmap, page)

    def MovePage(self, bitmap, target, source):
        """ Moves the source page to the position of the target page. 
        Returns TRUE on success, FALSE on failure.
        """
        return self.__lib.MovePage(bitmap, target, source)

    def GetLockedPageNumbers(self, bitmap, pages, count):
        """ Returns an array of page-numbers that are currently locked in memory.
        
            ----- Pass me an buffer int. Read the documentation for how to use me
        """
        return self.__lib.GetLockedPageNumbers(bitmap, pages, count)

    def InsertPage(self, bitmap, page, data):
        """ Inserts a new page before the given position in the bitmap.
        """
        self.__ctrlColorUsed()
        return self.__lib.InsertPage(bitmap, page, data)


    # ----------------- #
    #  Plugin functions #
    # ----------------- #
    
    def FIFSupportsReading(self, fif):
        """ Returns TRUE if the plugin belonging to the given 
            FREE_IMAGE_FORMAT can be used to
            load bitmaps, FALSE otherwise.
        """
        return self.__lib.FIFSupportsReading(fif)
    
    def GetFIFFromFilename(self, fileName):
        """This function takes a filename or a file-extension and returns 
        the plugin that can read/write files with that extension in 
        the form of a FREE_IMAGE_FORMAT identifier.
        """
        return self.__lib.GetFIFFromFilename(fileName)
    
    def FIFSupportsExportBPP(self, fif, bpp):
        """ Returns TRUE if the plugin belonging to the given 
            FREE_IMAGE_FORMAT can save a bitmap in the desired bit depth
        """
        return self.__lib.FIFSupportsExportBPP(fif, bpp)

    def FIFSupportsExportType(self, fif, type):
        """ Returns TRUE if the plugin belonging to the given 
            FREE_IMAGE_FORMAT can save a bitmap in the desired data type,
        """
        return self.__lib.FIFSupportsExportType(bitmap, fif, type)

    def FIFSupportsICCProfiles(self, fif):
        """ Returns TRUE if the plugin belonging to the given 
            FREE_IMAGE_FORMAT can load or save an ICC profile,
        """
        return self.__lib.FIFSupportsICCProfiles(bitmap, fif)

    def FIFSupportsWriting(self, fif):
        """ Returns TRUE if the plugin belonging to the given FREE_IMAGE_FORMAT 
        can be used to save bitmaps
        """
        return self.__lib.FIFSupportsWriting(bitmap, fif)    

    def GetFIFCount(self):
        """ Retrieves the number of FREE_IMAGE_FORMAT identifiers being 
            currently registered.
        """
        return self.__lib.GetFIFCount()

    def GetFIFDescription(self, fif):
        """ Returns a descriptive string that describes the bitmap formats 
            the given plugin can read and/or write.
        """
        return self.__lib.GetFIFDescription(bitmap)

    def GetFIFExtensionList(self):
        """ Returns a comma-delimited file extension list describing 
            the bitmap formats the given plugin can read and/or write.
        """
        return self.__lib.GetFIFExtensionList()

    def GetFIFFromFormat(self, format):
        """ Returns a FREE_IMAGE_FORMAT identifier from the format 
            string that was used to register the FIF.
        """
        return self.__lib.GetFIFFromFormat(format)

    def GetFIFFromMime(self, mime):
        """ Returns a FREE_IMAGE_FORMAT identifier from a MIME content type 
        string (MIME stands for Multipurpose Internet Mail Extension).
        """
        return self.__lib.GetFIFFromMime(mime)

    def GetFIFMimeType(self, fif):
        """ Given a FREE_IMAGE_FORMAT identifier, returns a MIME content type 
        string (MIME stands for Multipurpose Internet Mail Extension).
        """
        return self.__lib.GetFIFMimeType(fif)

    def GetFIFRegExpr(self, fif):
        """ Returns a comma-delimited file extension list describing 
            the bitmap formats the given plugin can read and/or write.
        """
        return self.__lib.GetFIFRegExpr(fif)
    
    def GetFormatFromFIF(self, fif):
        """ Returns the string that was used to register a plugin from the 
            system assigned FREE_IMAGE_FORMAT.
        """
        return self.__lib.GetFormatFromFIF(fif)

    def IsPluginEnabled(self, fif):
        """ Returns TRUE when the plugin is enabled, FALSE when the plugin is 
            disabled, -1 otherwise.
        """
        return self.__lib.IsPluginEnabled(fif)
    
    def RegisterExternalPlugin(self, path, format=0, description=0, extension=0, 
            regexpr = 0):
        """ Registers a new plugin to be used in FreeImage.
        """
        return self.__lib.RegisterExternalPlugin(path, format, description, 
                    extension, regexpr)

    def RegisterLocalPlugin(self, proc_address, format=0, description=0, extension=0, 
            regexpr = 0):
        """ Registers a new plugin to be used in FreeImage.
        """
        return self.__lib.RegisterLocalPlugin(proc_address, format, description, 
                    extension, regexpr)

    def SetPluginEnabled(self, fif, enabled):
        """ Enables or disables a plugin.
        """
        return self.__lib.SetPluginEnabled(fif, enabled)
    
    # ------------------- #
    #  Bitmap information #
    # ------------------- #

    def GetBackgroundColor(self, bitmap):
        """ Retrieves the file background color of an image.
        """
        self.__ctrlColorUsed()
        return self.__lib.GetBackgroundColor(bitmap)

    def GetFileType(self, fileName, size=False):
        """ Return the file type, like bmp, tiff, png
            This function return a value that are used in other functions
            that need the bitmap type
            Size it not used into the library
        """
        return self.__lib.GetFileType(fileName, size)

    def GetColorType(self, bitmap):
        """ Returm one of the FIC_* values
        """
        self.__ctrlColorUsed()
        return self.__lib.GetColorType(bitmap)
    
    def GetImageType(self, bitmap):
        """ Return the image type
        """
        self.__ctrlColorUsed()
        return self.__lib.GetImageType(bitmap)

    def GetWidth(self, bitmap):
        """ Return the bitmap width
        """
        self.__ctrlColorUsed()
        return self.__lib.GetWidth(bitmap)

    def GetHeight(self, bitmap):
        """ Return the bitmap height
        """
        self.__ctrlColorUsed()
        return self.__lib.GetHeight(bitmap)
    
    def GetBPP(self, bitmap):
        """ Return the depth
        """
        return self.__lib.GetBPP(bitmap)
    
    def GetDotsPerMeterX(self, bitmap):
        """ Return the horrizontal resolution
        """
        self.__ctrlColorUsed()
        return self.__lib.GetDotsPerMeterX(bitmap)

    def GetDotsPerMeterY(self, bitmap):
        """ Return the vertical resolution
        """
        self.__ctrlColorUsed()
        return self.__lib.GetDotsPerMeterY(bitmap)
    
    def SetDotsPerMeterX(self, bitmap, res):
        """ Set the horrizontal resolution
        """
        self.__ctrlColorUsed()
        return self.__lib.SetDotsPerMeterX(bitmap, res)

    def SetDotsPerMeterY(self, bitmap, res):
        """ Set the vertical resolution
        """
        self.__ctrlColorUsed()
        return self.__lib.SetDotsPerMeterY(bitmap, res)
    
    def GetColorsUsed(self, bitmap):
        """ Return the number of color used
        """
        self.__ctrlColorUsed()
        return self.__lib.GetColorsUsed(bitmap)
    
    def GetPalette(self, bitmap):
        """ Return a pointer to a palette struct 
        """
        self.__ctrlColorUsed()
        return self.__lib.GetPalette(bitmap)

    def GetBlueMask(self, bitmap):
        """ Returns a bit pattern describing the blue color component of a 
            pixel in a FIBITMAP.
        """
        self.__ctrlColorUsed()
        return self.__lib.GetBlueMask(bitmap)

    def GetRedMask(self, bitmap):
        """ Like up
        """
        self.__ctrlColorUsed()
        return self.__lib.GetRedMask(bitmap)
    
    def GetGreenMask(self, bitmap):
        """ Like up
        """
        self.__ctrlColorUsed()
        return self.__lib.GetGreenMask(bitmap)

    def GetInfoHeader(self, bitmap):
        """ Returns a pointer to the BITMAPINFOHEADER of the DIB-element in
            a FIBITMAP.
        """
        self.__ctrlColorUsed()
        return self.__lib.GetInfoHeader(bitmap)

    def GetLine(self, bitmap):
        """ Returns the width of the bitmap in bytes.
            See also: FreeImage_GetPitch.
        """
        self.__ctrlColorUsed()
        return self.__lib.GetLine(bitmap)

    def GetPitch(self, bitmap):
        """ Returns the width of the bitmap in bytes, rounded to the next 
            32-bit boundary, also known as pitch or stride or scan width.
        """
        self.__ctrlColorUsed()
        return self.__lib.GetPitch(bitmap)
    
    def SetTransparencyTable(self, bitmap, table, count):
        """ Set the bitmaps transparency table.
        """
        self.__ctrlColorUsed()
        return self.__lib.SetTransparencyTable(bitmap, table, count)

    def SetTransparent(self, bitmap, enabled):
        """ Tells FreeImage if it should make use of the transparency table 
        that may accompany a bitmap
        """
        self.__ctrlColorUsed()
        return self.__lib.SetTransparent(bitmap, enabled)

    def SetBackgroundColor(self, bitmap, bkcolor):
        """ Set the file background color of an image.
        """
        self.__ctrlColorUsed()
        return self.__lib.SetBackgroundColor(bitmap, bkcolor)
    
    def GetTransparencyCount(self, bitmap):
        """ Returns the number of transparent colors in a palletised bitmap
        """
        self.__ctrlColorUsed()
        return self.__lib.GetTransparencyCount(bitmap)

    def GetTransparencyTable(self, bitmap):
        """ Returns a pointer to the bitmaps transparency table.
        """
        self.__ctrlColorUsed()
        return self.__lib.GetTransparencyTable(bitmap)

    def HasBackgroundColor(self, bitmap):
        """ Returns TRUE when the image has a file background color
        """
        self.__ctrlColorUsed()
        return self.__lib.HasBackgroundColor(bitmap)
        
    # ----------------------- #
    #   Pixel access methods  #
    # ----------------------- #

    def GetPixelIndex(self, bitmap, x=0, y=0):
        """ Return the pixel index for the bitmap,
            default return the first at left top
        """
        self.__ctrlColorUsed()
        value = C.c_ubyte()
        xP = C.c_uint(x)
        yP = C.c_uint(x)
        self.__lib.GetPixelIndex( bitmap, xP, yP, C.pointer(value))
        return value.value
    
    def GetBits(self, bitmap):
        """ Returns a pointer to the data-bits of the bitmap.
        """
        self.__ctrlColorUsed()
        return self.__lib.GetBits(bitmap)
    
    def GetPixelColor(self, bitmap, x, y, value=None):
        """ Get the pixel color of a 16-, 24- or 32-bit image at position 
        (x, y), including range check (slow access).
        """
        if not value: value = C.POINTER(self.__RGBQUAD)
        self.__ctrlColorUsed()
        return self.__lib.GetPixelColor(bitmap, x, y, value), value

    def GetScanLine(self, bitmap, scanline):
        """ Returns a pointer to the start of the given scanline in 
            the bitmaps data-bits.
        """
        self.__ctrlColorUsed()
        return self.__lib.GetScanLine(bitmap, scanline)

    def SetPixelColor(self, bitmap, x, y, value=None):
        """ Set the pixel color of a 16-, 24- or 32-bit image at position 
            (x, y), including range check (slow access).
        """
        if not value: value = C.POINTER(self.__RGBQUAD)
        self.__ctrlColorUsed()
        return self.__lib.SetPixelColor(bitmap, x, y, value), value

    def SetPixelIndex(self, bitmap, x, y, value=None):
        """ Set the pixel index of a palettized image at position (x, y), 
            including range check (slow access).
        """
        if not value: value = C.POINTER(self.__RGBQUAD)
        self.__ctrlColorUsed()
        return self.__lib.SetPixelIndex(bitmap, x, y, value), value

    
    # -------------------------------------- #
    #   Conversion / Trasformation methods   #
    # -------------------------------------- #
    
    def Rescale(self, bitmap, dstWidth, dstHeight, filter=FILTER_BSPLINE, close=0):
        """ Return the bitmap rescaled with the parameter passed
        """
        self.__ctrlColorUsed()
        new_bitmap = self.__lib.Rescale(bitmap, int(dstWidth), int(dstHeight), filter)
        self.__closeIf(bitmap, close)
        return new_bitmap

    def Dither(self, bitmap, matrix=FID_FS):
        """ Transfom (out will be 1 bit per pixel) and return a copy of the bitmap
        """
        self.__ctrlColorUsed()
        return self.__lib.Dither(bitmap, matrix)
    
    def RotateClassic(self, bitmap, angle, close=0):
        """ Rotate the bitmap and return a new bitmap
        """
        self.__ctrlColorUsed()
        new_bitmap = self.__lib.RotateClassic(bitmap, C.c_double(angle))
        self.__closeIf(bitmap, close)
        return new_bitmap

    def RotateEx(self, bitmap, angle, x_shift, y_shift, x_origin, y_origin, 
                use_mask):
        """ This function performs a rotation and / or translation of an 8-bit 
            greyscale, 24- or 32-bit image, using a 3rd order (cubic) B-Spline.
        """
        self.__ctrlColorUsed()
        return self.__lib.RotateEx(bitmap,  angle, x_shift, y_shift, x_origin, 
                y_origin, use_mask)
    
    def ConvertTo4Bits(self, bitmap, close=0):
        """ Convert the bitmap to 4 bits depth
            and optionally close it
        """
        self.__ctrlColorUsed()
        new_bitmap = self.__lib.ConvertTo4Bits(bitmap)
        self.__closeIf(bitmap, close)
        return new_bitmap
    
    def ConvertTo8Bits(self, bitmap, close=0):
        """ Convert the bitmap to 8 bits depth
            and optionally close it
        """
        self.__ctrlColorUsed()
        new_bitmap = self.__lib.ConvertTo8Bits(bitmap)
        self.__closeIf(bitmap, close)
        return new_bitmap
    
    def ConvertTo16Bits555(self, bitmap, close=0):
        """ Convert the bitmap to 16/555 bits depth
            and optionally close it
        """
        self.__ctrlColorUsed()
        new_bitmap = self.__lib.ConvertTo16Bits555(bitmap)
        self.__closeIf(bitmap, close)
        return new_bitmap

    def ConvertTo16Bits565(self, bitmap, close=0):
        """ Convert the bitmap to 16/565 bits depth
            and optionally close it
        """
        self.__ctrlColorUsed()
        new_bitmap = self.__lib.ConvertTo16Bits565(bitmap)
        self.__closeIf(bitmap, close)
        return new_bitmap

    def ConvertTo32Bits(self, bitmap, close=0):
        """ Convert the bitmap to 64 bits depth
            and optionally close it
        """
        self.__ctrlColorUsed()
        new_bitmap = self.__lib.ConvertTo32Bits(bitmap)
        self.__closeIf(bitmap, close)
        return new_bitmap
    
    def ConvertToStandardType(self, bitmap, scale_linear = 1, close=0):
        """ Convert the passed image to a standard type
        """
        self.__ctrlColorUsed()
        new_bitmap = self.__lib.ConvertToStandardType(bitmap, scale_linear)
        self.__closeIf(bitmap, close)
        return new_bitmap
    
    def ColorQuantize(self, bitmap, quantize = FIQ_WUQUANT):
        """ Quantizes a high-color 24-bit bitmap to an 8-bit 
            palette color bitmap. The quantize parameter
            specifies the color reduction algorithm to be used:
        """
        self.__ctrlColorUsed()
        return self.__lib.ColorQuantize(bitmap, quantize)

    def ColorQuantizeEx(self, bitmap, quantize = FIQ_WUQUANT, PaletteSize = 256,
                        ReserveSize = 0, ReservePalette = None):
        """ FreeImage_ColorQuantizeEx is an extension to the 
        FreeImage_ColorQuantize function that provides additional options
        """
        self.__ctrlColorUsed()
        return self.__lib.ColorQuantizeEx(bitmap, quantize, PaletteSize, \
                        ReserveSize, ReservePalette)

    def ConvertFromRawBits(self, bits, width, height, pitch, bpp, red_mask, 
                            green_mask, blue_mask, topdown = False):
        """ Converts a raw bitmap somewhere in memory to a FIBITMAP.
        """
        self.__ctrlColorUsed()
        return self.__lib.ConvertFromRawBits(bits, width, height, pitch, bpp, 
                            red_mask, green_mask, blue_mask, topdown)
    def ConvertTo24Bits(self, bitmap):
        """
        """
        self.__ctrlColorUsed()
        return self.__lib.ConvertTo24Bits(bitmap)

    def ConvertToRawBits(self, bits, bitmap, pitch, bpp, red_mask, green_mask, 
            blue_mask, topdown = False):
        """ Converts a FIBITMAP to a raw piece of memory.
        """
        self.__ctrlColorUsed()
        return self.__lib.ConvertToRawBits(bits, bitmap, pitch, bpp, red_mask, 
                            green_mask, blue_mask, topdown)

    def ConvertToRGBF(self, bitmap):
        """ Converts a 24- or 32-bit RGB(A) standard image or a 48-bit 
            RGB image to a FIT_RGBF type image.
        """
        self.__ctrlColorUsed()
        return self.__lib.ConvertToRGBF(bitmap)

    def ConvertToType(self, bitmap, dst_type, scale_linear = True):
        """ Converts an image of any type to type dst_type.
        """
        self.__ctrlColorUsed()
        return self.__lib.ConvertToType(bitmap, dst_type, scale_linear )
    
    def Threshold(self, bitmap, T):
        """ Converts a bitmap to 1-bit monochrome bitmap using a 
            threshold T between [0..255].
        """
        self.__ctrlColorUsed()
        return self.__lib.Threshold(bitmap, T)
    
    def GetDIBSize(self, bitmap):
        """ Returns the size of the DIB-element of a FIBITMAP in memory
        """
        self.__ctrlColorUsed()
        return self.__lib.GetDIBSize(bitmap)
    
    
    # ---------------------------- #
    #  Color manipulation methods  #
    # ---------------------------- #
    
    def Invert(self, bitmap):
        """ Inverts each pixel data
        """
        self.__ctrlColorUsed()
        return self.__lib.Invert(bitmap)
    
    def AdjustBrightness(self, bitmap, percentage):
        """Adjusts the brightness of a 8-, 24- or 32-bit image by a 
            certain amount. 
        """
        self.__ctrlColorUsed()
        return self.__lib.AdjustContrast(bitmap, C.c_double(percentage) )

    def AdjustContrast(self, bitmap, percentage):
        """ Like up
        """
        self.__ctrlColorUsed()
        return self.__lib.AdjustContrast(bitmap, C.c_double(percentage))

    def AdjustCurve(self, bitmap, LUT, channel):
        """ Perfoms an histogram transformation on a 8-, 24- or 32-bit 
        image according to the values of a lookup table (LUT).
        """
        self.__ctrlColorUsed()
        return self.__lib.AdjustCurve(bitmap, LUT, channel)

    def AdjustGamma(self, bitmap, gamma):
        """ Performs gamma correction on a 8-, 24- or 32-bit image.
        """
        self.__ctrlColorUsed()
        return self.__lib.AdjustGamma(bitmap, C.c_double(gamma))

    def GetHistogram(self, bitmap, histo, channel = FICC_BLACK):
        """ Computes the image histogram.
        """
        self.__ctrlColorUsed()
        return self.__lib.GetHistogram(bitmap, histo, channel)
    
    # ---------------------------- #
    #  Channel processing methods  #
    # ---------------------------- #

    def GetChannel(self, bitmap, channel):
        """ Retrieves the red, green, blue or alpha channel of a 24- or 32-bit 
            image.
        """
        self.__ctrlColorUsed()
        return self.__lib.GetChannel(bitmap, channel)

    def SetChannel(self, bitmap, bitmap8, channel):
        """ Insert a 8-bit dib into a 24- or 32-bit image. dib8 and dib must 
            have the same width and height.
        """
        self.__ctrlColorUsed()
        return self.__lib.SetChannel(bitmap, bitmap8, channel)

    def GetComplexChannel(self, bitmap, channel):
        """ Retrieves the real part, imaginary part, magnitude or phase of a 
        complex image (image whose type is FIT_COMPLEX).
        """
        self.__ctrlColorUsed()
        return self.__lib.GetComplexChannel(bitmap, channel)
    
    def SetComplexChannel(self, bitmap, channel):
        """ Set the real or imaginary part of a complex image (image whose 
            type is FIT_COMPLEX).
        """
        self.__ctrlColorUsed()
        return self.__lib.SetComplexChannel(bitmap, channel)
    
    # ---------------------------- #
    #   Tags and metadata methods  #
    # ---------------------------- #
    
    def GetTagValue(self, tag):
        """ Return the tag value
        """
        return self.__lib.GetTagValue(tag)
    
    def GetTagCount(self, tag):
        """ Returns the number of components in the tag (in tag type units).
        """
        return self.__lib.GetTagCount(tag)
    
    def GetTagID(self, tag):
        """ Returns the tag field name (unique inside a metadata model).
        """
        return self.__lib.GetTagID(tag)
    
    def GetMetadata(self, mdModel, bitmap, key):
        """ Return metadata
        """
        return self.__lib.GetMetadata(mdModel, bitmap, key, data)
    
    def GetMetadataCount(self, tag):
        """ Returns the number of components in the tag
        """
        return self.__lib.GetMetadataCount(tag)

    def GetTagDescription(self, tag):
        """ Returns the tag description if available, returns NULL otherwise.
        """
        return self.__lib.GetTagDescription(tag)

    def TagToString(self, model, tag, make = None):
        """ Converts a FreeImage tag structure to a string that represents the 
            interpreted tag value.
        """
        return self.__lib.TagToString(model, tag, make)
    
   
    # ------------------------ #
    #   Help/internal methods  #
    # ------------------------ #

    def IsLittleEndian(self):
        """ This function returns TRUE if the platform running 
            FreeImage uses the Little Endian convention
        """
        return self.__lib.IsLittleEndian()


    def convertToMultiPage(self, filesIn, fileOut, 
                            outFormat=-1, outBpp=-1,
                            sameSize=False,  sameType=False,
                            keepSizeFrom=0, keepTypeFrom=0,
                            flags=False, convertToFax=0,
                            deleteOld=0):
        
        """ Function for create the fileOut bitmap with fileIn list
            and optionally resize and convert the format of the bitmaps
            If you pass me format, I keep that like output format
            
            filesIn can be a list or tuple with (fileName, type) or only
            the filesNames, so I *try* to get the type
            
            deleteOld is used to tell me to delete the old files
        """
        
        #Try if we can read/write access to the output dir
        outDir = os.path.abspath( os.path.split(fileOut)[0] )
        try:
            file = tempfile.TemporaryFile(dir=outDir)
            time.sleep(0.1)
            file.close()
        except OSError:
            return 5, 'Error: cannot have read/write access to %s' % outDir

        #Delete the old file
        if os.path.exists(fileOut):
            try: os.remove(fileOut)
            except OSError: return 4, 'Error: cannot delete file %s' % fileOut
        
        #If convert to fax, I already know the type and the Bpp
        if convertToFax:
            outFormat = FIF_TIFF
            flags=TIFF_CCITTFAX4
            outBpp=1
        
        #Find the type
        elif outFormat == -1 and sameType:
            
            #Get the format
            outFormat = self.genericFileDeducer(filesIn[keepTypeFrom])

            #Get the format
            bmpForBpp = self.genericLoader(filesIn[keepTypeFrom])
            
            if not bmpForBpp:
                return 1, 'Error on type on outFormat'
            
            #Get the depth
            outBpp = self.GetBPP(bmpForBpp)
            
            self.Unload(bmpForBpp)
            
        elif outFormat == -1 and not sameType:
            return 1, 'Error: Specify a format, please'
        
        #Find the size
        if sameSize and not convertToFax:
            bmpForType = self.genericLoader(filesIn[keepSizeFrom])
            if not bmpForType:
                return 1, 'Error: no type on sameSize'
            
            outW, outH = self.getSize(bmpForType)
            self.Unload(bmpForType)
        
        #The output bitmap
        outBmp = self.OpenMultiBitmap(outFormat, fileOut, True, flags=flags)
        if not outBmp:
            return 2, 'Error: I cannot open the multibitmap for write. Filename: %s , Format: %s, flags: %s' % \
                        ( fileOut, outFormat, flags)
        
        multiPageSplit = list()
        for ismultipage in filesIn:
            multiPageSplit += self.convertToSinglePages(ismultipage, FIF_PNG, 
                                '.png') #, returnOpenBitmap=1)
        
        for num, bmp in enumerate(multiPageSplit):
            
            #Control the types of passed paramenter
            if type(bmp) in ( list(), tuple() ):
                bmpFileName, bmpType = bmp
            else:
                bmpFileName = bmp
                bmpType = self.genericFileDeducer(bmpFileName)
                if bmpType == FIF_UNKNOWN:
                    return 1, 'Error: no find type on open %s' % bmpFileName
            
            #Load the bitmap
            bmpWork = self.Load(bmpType, bmpFileName)

            if not bmpWork:
                return 2, 'Error: I cannot open the bitmap %s' % bmpFileName

            #Size control
            if convertToFax: 
                bmpWork = self.convertToA4Fax(bmpWork)
            elif sameSize and num != keepSizeFrom:
                bmpWork = self.Rescale(bmpWork, outW, outH, close=1)
            
            # Type/depth control
            if sameType and num != keepTypeFrom and not convertToFax:
                bmpWork = self.setBppFromBpp(outBpp, bmpWork)
            
            self.AppendPage(outBmp, bmpWork)
            self.Unload(bmpWork)
            if deleteOld:
                try: 
                    if os.path.exists(bmpFileName): os.remove(bmpFileName)
                except OSError: pass
        
        self.CloseMultiBitmap(outBmp, flags)
        
        # Delete if it is request. I delete only the files that are created 
        # when I split the multipage bitmap that come-in
        for page in multiPageSplit:
            if page in filesIn: continue
            try: os.remove(page)
            except OSError: pass
        
        return 0, 'All ok!! File saved on %s' % fileOut

    def convertToA4Fax(self, bitmap):
        """ Convert the passed image to 1728 x 2210 pixel
            (206 x 196) DPI that is need when work with fax
        """

        W, H = self.getDotDPI(bitmap)
        X, Y = self.getSize(bitmap) 

        #Control if the current file has the right resolution and size
        if not ( (W == 204 and H == 196) and (Y > X)):
            #Convert to 4 bits
            bitmap = self.setBppFromBpp(8, bitmap)
            bitmap = self.Rescale(bitmap, 1728, 2210, close=1)
        
        bitmap = self.ConvertToStandardType(bitmap, close=1)
        
        #Convert to 4 bits
        #self.Save(FIF_TIFF, bitmap, './aaaaa_bef1%s.tif'% self.__num)
        bitmap = self.setBppFromBpp(1, bitmap)
        
        #self.Save(FIF_TIFF, bitmap, './aaaaa_aft%s.tif'% self.__num)
        self.convertToMinIsWhite(bitmap)
        
        self.setDotDPI(bitmap, (204, 196))        
        
        #self.Save(FIF_TIFF, bitmap, './aaaaa_aft_2_%s.tif'% self.__num)
        self.__num+=1
        return bitmap

    def convertToMinIsWhite(self, bitmap):
        """ Invert the bitmap colors and the palette
            from minIsBlack to minIsWhite
        """
        if not self.GetColorType(bitmap) == FIC_MINISBLACK: return

        self.Invert(bitmap)
        point = self.GetPalette(bitmap)
        
        for i in range(self.GetColorsUsed(bitmap)):
            cont = point[i]
            cont.rgbRed	  = 255 - cont.rgbRed
            cont.rgbGreen = 255 - cont.rgbGreen
            cont.rgbBlue  = 255 - cont.rgbBlue
    
    def convertToSinglePages(self, multiPageName, outputFormat, outputExt, 
                        outputName=None, flags=None, bitmapType=-1, 
                        returnOpenBitmap=0, outputDir=None):
        """ Return the multiPage splitted into single pages
            The returned name are "outputNameXXX.outputExt"
            Where XXX is a progressive number, started from 0
            If outputName isn't set, I create a new one name with tempfile
            
            If returnOpenBitmap is set, I return the opened bitmaps, 
            loaded into FITBITMAP structures
        """
        
        #Create the output file names if not present and delete it
        if not (outputName or returnOpenBitmap):
            if outputDir: fd, outputName = tempfile.mkstemp(dir=outputDir)
            else: fd, outputName = tempfile.mkstemp()
            f = os.fdopen(fd, 'w'); f.close()
            try: os.remove(outputName)
            except OSError: pass
            
        if bitmapType == -1:
            bitmapType = self.genericFileDeducer(multiPageName)
        
        if bitmapType == FIF_UNKNOWN:
            return 1, 'Cannot identify image type, %s' % multiPageName
        
        if bitmapType not in (FIF_TIFF, FIF_ICO, FIF_GIF) or \
            (self.isMultiPage(multiPageName, bitmapType) == 1 and \
                            outputFormat == bitmapType):
            if returnOpenBitmap:
                return [self.genericLoader(multiPageName),]
            else:
                return [multiPageName,]
        
        #Open multibitmap
        bitmapMulti = self.OpenMultiBitmap(bitmapType, multiPageName, False)
        if not bitmapMulti:
            return 2, 'This image is not a multiImage'
        fileNameList = list()
        
        #Iternate over the multipage bitmap
        for num in xrange( self.GetPageCount(bitmapMulti) ):
            
            bmpPage = self.LockPage(bitmapMulti, num)
            
            #If I have to return a loaded bitmap, clone it
            if returnOpenBitmap:
                newBitmap = self.Clone(bmpPage)
                fileNameList.append(newBitmap)
            
            #else, save it to a new file
            else:
                outputName = os.path.splitext(outputName)[0]
                fileNameOut = '%s%s%s' % (outputName, num, outputExt)
                valRetSave = self.Save(outputFormat, bmpPage, fileNameOut, flags)
                if not valRetSave: return 'Error: problems on save: %s' % fileNameOut
                fileNameList.append(fileNameOut)
            
            self.UnlockPage(bitmapMulti, bmpPage)
        
        #Close and return the list
        self.CloseMultiBitmap(bitmapMulti)
        
        return  fileNameList
    
    def setBppFromBitmap(self, bitmapFrom, bitmapTo, closeTo=1):
        """ Change the bitmapTo depth to the bitmapFrom depth
            If closeTo if true, if close I return the new bitmap and
            the close the oldBitmap one
        """
        depthF = self.GetBPP(bitmapFrom)
        return self.setBppFromBpp(depthF, bitmapFrom, closeTo)
        
    
    def setBppFromBpp(self, BppFrom, bitmapTo, closeTo=1):
        """ Change the bitmapTo depth to the bitmapFrom depth
            If closeTo if true, if close I return the new bitmap and
            the close the oldBitmap one
        """
        
        #If the bitmap has alreasy the right depth, return
        if self.GetBPP(bitmapTo) == BppFrom: 
            #print "setBppFromBpp already the right depth"
            return bitmapTo
        
        if BppFrom == 1:
            action = self.Dither
        elif BppFrom == 4:
            action = self.ConvertTo4Bits
        elif BppFrom == 8:
            action = self.ConvertTo8Bits
        elif BppFrom == 16:
            action = self.ConvertTo16Bits555
        elif BppFrom == 16:
            action = self.ConvertTo32Bits
        else:
            #I don't know how to convert the bitmap
            return 0
        
        new_bitmapTo = action(bitmapTo)
        self.__closeIf(bitmapTo, closeTo)
        return new_bitmapTo
    
    def getSize(self, bitmap):
        """ Help method. Return a tuple with (widht, height)
        """
        return ( self.__lib.GetWidth(bitmap), self.__lib.GetHeight(bitmap) )
    
    def getDotDPI(self, bitmap):
        """ Return the bitmap DPIs
        """
        W = int( round(self.GetDotsPerMeterX(bitmap) / 39.37) )
        H = int( round(self.GetDotsPerMeterY(bitmap) / 39.37 ) )
        return W, H

    def setDotDPI(self, bitmap, res):
        """ Return the bitmap DPIs
        """
        W, H = res
        return self.SetDotsPerMeterX(bitmap, int( round( W * 39.37)) ), \
               self.SetDotsPerMeterY(bitmap, int( round( H * 39.37)) )

    def isMultiPage(self, fileName, fileType=False):
        """ Return if the fileName passed is a multipage file
            the number of page is returned
        """
        if not fileType:
            fileType = self.genericFileDeducer(fileName)

        if fileType == FIF_UNKNOWN:
            return 1, 'Image type error'
        bitmapMulti = self.OpenMultiBitmap(fileType, fileName, False)
        if not bitmapMulti:
            return 2, 'Cannot open image'
        pageN = self.GetPageCount(bitmapMulti)
        self.CloseMultiBitmap(bitmapMulti)
        return pageN

    def genericLoader(self, fileName, flag=0):
        """ Check the file signature and deduce its format
            (the second argument is currently not used by FreeImage)
            return the bitmap loaded
        
            --- Thanks to the C++ code found on FreeImage examples
        """
        
        fif = self.genericFileDeducer(fileName)
        
        #If a type is found, return the loaded image
        if (fif != FIF_UNKNOWN) and self.FIFSupportsReading(fif):
            dib = self.Load(fif, fileName, flag);
            return dib
        #else return none
        return None
    
    def genericFileDeducer(self, fileName):
        """ Check the file signature and deduce its format
            (the second argument is currently not used by FreeImage)
            return the type
            
            --- Thanks to the C++ code found on FreeImage examples
        """
        
        fif = FIF_UNKNOWN
        try:
            fif = self.GetFileType(fileName)
            #I don't found any time
            if fif == FIF_UNKNOWN:
                fif = self.GetFIFFromFilename(fileName)
            return fif
        except:
            return fif
        
    def loadAndSave(self, fileNameLoad, dirNameSave, typeSave, fileLoadType=None):
        """ Load the passed filename and save it with the type
            If dirNameSave is a directory, I create a new filename with tempfile,
            else I save the new bitmap with that file name
            For typeSave argoument, if you can pass me a value (string) present into
            extToType dictionary, I see the type, flags and extension from there,
            else, pass me a list/tuple with this three values            
        """
        
        if typeSave in extToType.keys():
            typeForSave, flagsForSave, extForSave = extToType[typeSave]
        else:
            typeForSave, flagsForSave, extForSave = typeSave
        
        if not fileLoadType:
            bitmap = self.genericLoader(fileNameLoad, flagsForSave)
        else:
            bitmap = self.Load(fileLoadType, fileNameLoad, flagsForSave)
        
        if not bitmap:
            return 'Error, cannot open %s' % fileNameLoad
        
        if os.path.isdir(dirNameSave):
            fileNameSave = tempfile.mkstemp(dir=dirNameSave, 
                            suffix=extForSave)[1]
        else:
            dirName, prefixName = os.path.split(dirNameSave)
            if prefixName.find(".") != -1:
                prefixName = os.path.splitext(prefixName)[0]
            fileNameSave = tempfile.mkstemp(dir=dirName,  prefix = prefixName,
                            suffix=extForSave)[1]
            
        #control the type
        if typeForSave == FIF_TIFF and flagsForSave in (TIFF_CCITTFAX3, TIFF_CCITTFAX4):
            new_bmp = self.setBppFromBpp(1, bitmap)
            bitmap = new_bmp
       
        self.Save(typeForSave, bitmap, fileNameSave, flagsForSave)
        
        return fileNameSave
    
    def getStatus(self):
        """ Return the library status 
        """
        return self.__lib.getStatus()

    # ---------------- #
    #  Private methods #
    # ---------------- #
    
    def __ctrlColorUsed(self):
        """
        """
        caller = "FreeImage_%s" % sys._getframe(1).f_code.co_name
        locals = sys._getframe(1).f_locals
        if not locals.has_key('bitmap'): return
        bitmap = locals['bitmap']
        for funct in self.__lstFunction:
            functName = funct[0]
            
            #Colour depth Control 
            if caller == functName and len(funct) > 2:
                colorUsed = funct[2]
                bpp = self.GetBPP(bitmap)
                if not bpp in colorUsed:
                    
                    #Unload at least the bitmap and the library
                    try: self.Unload(bitmap)
                    except: pass
                    try: self.DeInitialise()
                    except: pass
                    
                    #Raise an exception
                    raise FreeImagePy_ColorWrong, \
                        'Wrong color %s in function: %s. I can use: %s' % \
                        (bpp, functName, colorUsed)
                break
            
            #No control need, go out and save cpu time
            elif caller == functName: 
                break
        
    def __ctrlInit(self):
        """ Control if I call the init
        """
        pass
    
    def __closeIf(self, bitmap, close):
        """ Help method for internal close arg
            A lot of my functions need a simple
            but very annoing code, so I was create...
            
            My use:  self.__closeIf(bitmap, close)
        """
        
        
        if close:
            self.Unload(bitmap)
